from analyses.analysisbase import tests as base_tests

from analyses.deviations import models as app_models

from django.contrib.auth.models import User


from analyses.analysisbase.tests.scenario import print_tree
from analyses.deviations import evaluation
from django.test import TestCase
from installations import models as installations_models

from utils import tests as utils_tests
from utils.trees import models as trees_models
from utils.trees import views as trees_views
from utils.versioning.models import WorkVersion, Version

from django.conf import settings
from django.db import connection


class PermissionsMixIn(object):
    def setPermissions(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('edit_versions','versioning','version'),
            ('display_deviationscenarios','deviations','deviationscenario'),
            ('edit_deviationscenarios','deviations','deviationscenario'),
            ('display_installations','installations','section'),
            ('display_deviationsuggestionlists','deviations','deviationsuggestionlist'),
            ('edit_deviationsuggestionlists','deviations','deviationsuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_deviationscenarios','deviations','deviationscenario'),
            ('display_installations','installations','section'),
            ('display_deviationsuggestionlists','deviations','deviationsuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class SettingsMixIn(object):
    sc_model = app_models.DeviationScenario
    ev_model = app_models.Cause
    pl_model = app_models.ProtectionLayer
    me_model = app_models.DeviationMeasure
    plme_model = app_models.DeviationProtectionLayer_Measure
    ap_model = app_models.AttentionPoint
    sl_model = app_models.DeviationSuggestionlist
    apsl_model = app_models.DeviationAPSuggestionlist
    tree_model = app_models.CauseTreeElement
    namespace = "deviations"


class Deviations_SuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.SuggestionlistTest):
    pass

class Deviations_ScenarioTest(PermissionsMixIn, SettingsMixIn, base_tests.ScenarioTest):
    def test_extra_causetree_views(self):
        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class togglecause(utils_tests.edit_TestDefinition):
            description = 'toggle cause'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "togglecause|%s;" % self.elc1.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                testcase.assertEquals(self.test_class.ev_model.objects.get(v_id=self.test_class.elc1.v_id, version__isnull=True).condition, False)
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                if allowed:
                    testcase.assertEquals(self.test_class.ev_model.objects.get(v_id=self.test_class.eventc1.v_id, version__isnull=True).condition, True)
                else:
                    testcase.assertEquals(self.test_class.ev_model.objects.get(v_id=self.test_class.eventc1.v_id, version__isnull=True).condition, False)
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.ev_model.objects.get(v_id=self.test_class.eventc1.v_id, version__isnull=True)
                    x.condition = False
                    x.save(defaultuser)

        tests = [
            togglecause(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_extra_scenario_views(self):
        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class scenario_lopa(utils_tests.TestDefinition):
            description = 'display scenario tab lopa'
            url = base_url + '%s/lopa/' % self.scenario.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/lopa/edit/' % self.scenario.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.scenario.v_id, 'users': []},
            ]

        class scenario_lopa_edit(utils_tests.edit_TestDefinition):
            description = 'edit scenario tab lopa'
            url = base_url + '%s/lopa/edit/' % self.scenario.v_id
            users = [1,]
            data = {
                "use_LOPA": True,
                "target_frequency": '0.001',
                "management-referer": base_url + '%s/lopa/' % self.scenario.v_id
            }
            redirect_url = base_url + '%s/lopa/' % self.scenario.v_id


class Deviations_EventTest(PermissionsMixIn, SettingsMixIn, base_tests.EventTest):
    pass

class Deviations_VersioningTest(PermissionsMixIn, SettingsMixIn, base_tests.VersioningTest):
    pass

class Deviations_ProtectionLayerTest(PermissionsMixIn, SettingsMixIn, base_tests.ProtectionLayerTest):
    def test_protectionlayer_overrideable_views(self):
        base_url = '/analyses/%s/protectionlayers/' % self.namespace
        parent_url = '/analyses/%s/scenarios/%s/causetree/' % (self.namespace, self.scenario.v_id)
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class protectionlayer_add(utils_tests.add_TestDefinition):
            description = 'add protectionlayer'
            url = '/analyses/%s/scenarios/%s/protectionlayers/add/' % (self.namespace, self.scenario.v_id)
            users = [1,]
            data = {
                "name": "bogus",
                "description": "",
                "reltype": 1,
                "management-referer": parent_url
            }
            redirect_url = parent_url
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.pl_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class protectionlayer_edit(utils_tests.edit_TestDefinition):
            description = 'edit protectionlayer'
            url = base_url + '%s/edit/' % self.protectionlayer.v_id
            users = [1,]
            data = {
                "name": "Interlock",
                "description": "",
                "reltype": 1,
                "management-referer": base_url + '%s/' % self.protectionlayer.v_id
            }
            redirect_url = base_url + '%s/' % self.protectionlayer.v_id

        tests = [
            protectionlayer_add(self),
            protectionlayer_edit(self),
        ]

        utils_tests.run_tests(self, users, tests)

class Deviations_MeasureTest(PermissionsMixIn, SettingsMixIn, base_tests.MeasureTest):
    pass

class Deviations_APSuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.APSuggestionlistTest):
    pass

class Deviations_AttentionPointTest(PermissionsMixIn, SettingsMixIn, base_tests.AttentionPointTest):
    pass

class DeviationDependencyTest(PermissionsMixIn, TestCase):
    fixtures = ['dummyusers', 'installations', 'deviations']
    def setUp(self):
        self.setPermissions()

    def test_dependency_views(self):

        base_url = '/analyses/deviations/dependencies/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class dependency(utils_tests.TestDefinition):
            description = 'display dependency'
            url = base_url + '1/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': []},
                {'url': base_url + '1/delete/', 'users': [1,]},
            ]

        class dependency_add(utils_tests.add_TestDefinition):
            description = 'add dependency'
            url = '/analyses/deviations/scenarios/1/dependencies/add/'
            users = [1,]
            data = {
                "element1": 6,
                "element2": 5,
                "management-referer": '/analyses/deviations/scenarios/1/causetree/'
            }
            redirect_url = 'analyses/deviations/scenarios/1/causetree/'
            def posttest(self, testcase, allowed):
                if allowed:
                    app_models.Dependency.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class dependency_delete(utils_tests.delete_TestDefinition):
            description = 'delete dependency'
            users = [1,]
            redirect_url = '/analyses/deviations/scenarios/1/causetree/'
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = app_models.Dependency(element1_id=6, element2_id=5)
                newitem.save(defaultuser)
                self.pk = newitem.pk
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    app_models.Dependency.objects.get(pk=self.pk).delete(defaultuser, override=True)

        tests = [
            dependency(self),
            dependency_add(self),
            dependency_delete(self),
        ]

        utils_tests.run_tests(self, users, tests)

class DeviationLopaPathTest(PermissionsMixIn, TestCase):
    fixtures = ['dummyusers', 'installations', 'deviations']
    def setUp(self):
        self.setPermissions()

    def test_lopapath_views(self):

        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class lopa_paths(utils_tests.tablelist_TestDefinition):
            description = 'list of deviation lopa paths'
            url = '/analyses/deviations/lopapaths/'
            users = [1,2,]
            rowcount = 2

        class lopa_path(utils_tests.TestDefinition):
            description = 'display deviation scenario lopa path details'
            users = [1,2,]
            url = '/analyses/deviations/lopapath/1/'
            def pretest(self, testcase, allowed):
                paths = app_models.DeviationLopaPath.objects.all()
                if paths:
                    self.url = '/analyses/deviations/lopapath/%s/' % paths[0].id

        class lopa_path_report(utils_tests.TestDefinition):
            description = 'pdf report deviation scenario lopa path details'
            users = [1,2,]
            url = '/analyses/deviations/lopapath/1/pdf/'
            def pretest(self, testcase, allowed):
                paths = app_models.DeviationLopaPath.objects.all()
                if paths:
                    self.url = '/analyses/deviations/lopapath/%s/pdf/' % paths[0].id

        tests = [
            lopa_paths(self),
            lopa_path(self),
            lopa_path_report(self),
        ]

        utils_tests.run_tests(self, users, tests)

class DeviationsLopaDirtyTest(PermissionsMixIn, TestCase):
    fixtures = ['dummyusers', 'installations', 'deviations']
    def setUp(self):
        self.setPermissions()
    def test_scenario(self):
        user = User.objects.get(pk=1)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        s1.target_frequency = 0.001
        s1.save(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

        # version this
        v1 = Version(name='v1')
        v1.save(user)

        # check changing lopa_dirty does not make the item version dirty
        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.dirty, False)
        self.assertEquals(s1.lopa_dirty, True)

        s1.set_lopa_dirty(False)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.dirty, False)
        self.assertEquals(s1.lopa_dirty, False)

        # check that versioned items are lopa_dirty
        # version this
        v2 = Version(name='v1')
        v2.save(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=v1)
        self.assertEquals(s1.dirty, False)
        self.assertEquals(s1.lopa_dirty, True)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=v2)
        self.assertEquals(s1.dirty, False)
        self.assertEquals(s1.lopa_dirty, True)

    def test_treeelement(self):
        user = User.objects.get(pk=1)

        # edit tree
        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        app_models.CauseTreeElement.objects.get(v_id=3, version__isnull=True).move_to(
            user,
            app_models.CauseTreeElement.objects.get(v_id=2, version__isnull=True), 'last-child')

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

        # delete tree element
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        c1 = app_models.CauseTreeElement.objects.get(v_id=3, version__isnull=True)
        c1.delete(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

    def test_cause(self):
        user = User.objects.get(pk=1)

        # edit cause
        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        c1 = app_models.Cause.objects.get(v_id=1, version__isnull=True)
        c1.name = 'whatever'
        c1.save(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

        # delete cause
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        c1 = app_models.Cause.objects.get(v_id=1, version__isnull=True)
        c1.delete(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

    def test_protectionlayer(self):
        user = User.objects.get(pk=1)

        # edit pl
        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        c1 = app_models.ProtectionLayer.objects.get(v_id=1, version__isnull=True)
        c1.name = 'whatever'
        c1.save(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

        # delete pl
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        c1 = app_models.ProtectionLayer.objects.get(v_id=1, version__isnull=True)
        c1.delete(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

    def test_dependency(self):
        user = User.objects.get(pk=1)

        # adding dependency

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        d1 = app_models.Dependency(
            element1 = app_models.CauseTreeElement.objects.get(v_id=3, version__isnull=True),
            element2 = app_models.CauseTreeElement.objects.get(v_id=5, version__isnull=True)
        )
        d1.save(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)

        # delete dependency
        s1.lopa_dirty = False
        s1.save(user)
        self.assertEquals(s1.lopa_dirty, False)

        d1 = app_models.Dependency.objects.get(element1=app_models.CauseTreeElement.objects.get(v_id=3, version__isnull=True), version__isnull=True)
        d1.delete(user)

        s1 = app_models.DeviationScenario.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        self.assertEquals(s1.lopa_dirty, True)


class DeviationsEvaluationTestCase(PermissionsMixIn, TestCase):
    fixtures = ['dummyusers', 'installations', 'lopa_test']
    def setUp(self):
        self.setPermissions()
    def test_evaluation(self):
        user = User.objects.get(pk=1)

        # scenario 1:
        # event only
        # LOC
        #  - event (frequency 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=1)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.1), True)

        # scenario 2:
        # single PFD PL
        # LOC
        #  - PL1 (PFD 0.2)
        #    - event (frequency 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=2)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.02), True)

        # scenario 3:
        # double PFD PL
        # LOC
        #  - PL2 (PFD 0.3)
        #    - PL1 (PFD 0.2)
        #      - event (frequency 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=3)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.006), True)

        # scenario 4:
        # single FF PL
        # LOC
        #  - PL1 (FF 0.1)
        #    - condition (prob: 0.5)
        scenario = app_models.DeviationScenario.objects.get(pk=4)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.1), True)

        # scenario 5:
        # single FF PL, no condition
        # LOC
        #  - PL1 (FF 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=5)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.1), True)

        # scenario 6:
        # FF PL and PFD PL
        # LOC
        #  - PL2 (PFD 0.3)
        #    - PL1 (FF 0.2)
        #      - condition (prob: 0.5)
        scenario = app_models.DeviationScenario.objects.get(pk=6)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.06), True)

        # scenario 7:
        # OR events only
        # LOC
        #  - event1 (freq 0.1)
        #  OR
        #  - event2 (freq 0.2)
        scenario = app_models.DeviationScenario.objects.get(pk=7)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 2)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.1), True)
        self.assertEquals(evaluation.feq(data[1].frequency, 0.2), True)

        # scenario 8:
        # OR events only
        # LOC
        #  - PL1 (PFD 0.2)
        #    - event1 (freq 0.1)
        #  OR
        #  - PL2 (PFD 0.3)
        #    - event2 (freq 0.01)
        scenario = app_models.DeviationScenario.objects.get(pk=8)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 2)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.02), True)
        self.assertEquals(evaluation.feq(data[1].frequency, 0.003), True)

        # scenario 9:
        # PL and two branches
        # LOC
        #  - PL3 (PFD 0.04)
        #    - intermediate event
        #      - PL1 (PFD 0.2)
        #        - event1 (freq 0.1)
        #      OR
        #      - PL2 (PFD 0.3)
        #        - event2 (freq 0.01)
        scenario = app_models.DeviationScenario.objects.get(pk=9)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 2)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.0008), True)
        self.assertEquals(evaluation.feq(data[1].frequency, 0.00012), True)

        # scenario 10:
        # manual freq
        # LOC
        #  - manual event (freq 0.01)
        #    - PL1 (PFD 0.2)
        #      - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=10)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.01), True)

        # scenario 11:
        # event AND condition
        # LOC
        #  - condition (prob: 0.5)
        #  AND
        #  - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=11)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.05), True)

        # scenario 12:
        # AND: complex condition
        # LOC
        #  - condition1
        #    - condition2
        #      - condition3 (prob: 30%)
        #  AND
        #  - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=12)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.03), True)

        # scenario 16:
        # manual freq with condition
        # LOC
        #  - event
        #    - PL1 (PFD 0.2)
        #      - event (freq 0.1)
        #        - condition
        scenario = app_models.DeviationScenario.objects.get(pk=16)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.02), True)

        # scenario 17:
        # error: PL without child needs FF, nothing set
        # LOC
        #  - event
        #    - PL1
        scenario = app_models.DeviationScenario.objects.get(pk=17)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 18:
        # error: PL without child needs FF, PFD set
        # LOC
        #  - event
        #    - PL1 (PFD=0.2)
        scenario = app_models.DeviationScenario.objects.get(pk=18)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 19:
        # error: PL max 1 child
        # LOC
        #  - event
        #    - PL1 (PFD=0.2)
        #       - event1
        #       - OR
        #       - event2
        scenario = app_models.DeviationScenario.objects.get(pk=19)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 21:
        # error: no mixing in OR
        # LOC
        #  - cause
        #    - event1
        #    - OR
        #    - condition1
        scenario = app_models.DeviationScenario.objects.get(pk=21)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 22:
        # error: AND needs 1 event - no events
        # LOC
        #  - cause
        #    - condition1
        #    - AND
        #    - condition2
        scenario = app_models.DeviationScenario.objects.get(pk=22)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 23:
        # error: AND needs 1 event - two events
        # LOC
        #  - cause
        #    - event1
        #    - AND
        #    - event2
        scenario = app_models.DeviationScenario.objects.get(pk=23)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 24:
        # error: event with condition child
        # LOC
        #  - event
        #    - condition
        scenario = app_models.DeviationScenario.objects.get(pk=24)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 25:
        # error: event with condition children only
        # LOC
        #  - event
        #    - condition1
        #    - OR
        #    - condition2
        scenario = app_models.DeviationScenario.objects.get(pk=25)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 26:
        # error: condition with event child
        # LOC
        #  - condition
        #    - event
        #  - AND
        #  - event
        scenario = app_models.DeviationScenario.objects.get(pk=26)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 27:
        # error: condition with event child in subgate
        # LOC
        #  - condition
        #    - event
        #    - AND
        #    - condition
        #  - AND
        #  - event
        scenario = app_models.DeviationScenario.objects.get(pk=27)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 28:
        # AND conditions results in condition
        # LOC
        #  - condition
        #    - condition1 (prob 20%)
        #    - AND
        #    - condition2 (Prob 30%)
        #  - AND
        #  - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=28)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.006), True)

        # scenario 29:
        # Conditions may not have event children, not even inside a gate
        # LOC
        #  - condition
        #    - event1
        #    - OR
        #    - event2
        #  - AND
        #  - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=29)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 1)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 0)

        # scenario 30:
        # availability
        # LOC
        #  - condition
        #    - pl avail (avail: 80%)
        #  - AND
        #  - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=30)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.02), True)

        # scenario 31:
        # availability with child
        # LOC
        #  - condition
        #    - pl avail (avail: 80%)
        #      - condition
        #  - AND
        #  - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=31)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.02), True)

        # scenario 33:
        # PL leading to condition: ignore FF or PFD
        # LOC
        #  - condition1
        #    - PL1 (PFD 0.2)
        #      - condition2 (prob: 0.3)
        #  AND
        #  - event (freq 0.1)
        scenario = app_models.DeviationScenario.objects.get(pk=33)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.03), True)

        # scenario 34:
        # Dependency two PL's
        # LOC
        #  - event
        #    - PL1 (PFD 0.01) (1)
        #      - event
        #        - PL2 (PFD 0.1) (1)
        #          - event
        scenario = app_models.DeviationScenario.objects.get(pk=34)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.01), True)

        # scenario 35:
        # Dependency two PL's
        # LOC
        #  - event
        #    - PL1 (PFD 0.1) (1)
        #      - event
        #        - PL2 (PFD 0.01) (1)
        #          - event
        scenario = app_models.DeviationScenario.objects.get(pk=35)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.01), True)

        # scenario 36:
        # Dependency two PL's: PFD and FF
        # LOC
        #  - event
        #    - PL1 (PFD 0.2) (1)
        #      - event
        #        - PL2 (FF 0.01) (1)
        #          - event
        scenario = app_models.DeviationScenario.objects.get(pk=36)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.01), True)

        # scenario 37:
        # Dependency two PL's: PFD and FF
        # LOC
        #  - event
        #    - PL1 (FF 0.01) (1)
        #      - event
        #        - PL2 (PFD 0.2) (1)
        #          - event
        scenario = app_models.DeviationScenario.objects.get(pk=37)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 1)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.01), True)

        # scenario 38:
        # Dependency PL and cause
        # LOC
        #  - event
        #    - PL1 (PFD 0.01) (1)
        #      - event
        #        - OR
        #          - event 1 (freq 0.03)
        #          - event 2 (freq 0.02) (1)
        scenario = app_models.DeviationScenario.objects.get(pk=38)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        self.assertEquals(len(errors), 0)
        data = scenario.deviationlopapath_set.all()
        self.assertEquals(len(data), 2)
        self.assertEquals(evaluation.feq(data[0].frequency, 0.0003), True)
        self.assertEquals(evaluation.feq(data[1].frequency, 0.02), True)

    def test_caching(self):
        """
        Assert that a second evaluation of a scenario does not need to hit the database
        """
        user = User.objects.get(pk=1)

        olddebug = settings.DEBUG
        settings.DEBUG = True

        scenario = app_models.DeviationScenario.objects.get(pk=1)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        qu1 = len(connection.queries)

        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        qu2 = len(connection.queries)
        self.assertEquals(qu2 - qu1, 0)

        settings.DEBUG = olddebug

    def test_caching_public(self):
        """
        Assert that evaluating a different version of a scenario retriggers the evaluation
        """
        user = User.objects.get(pk=1)

        olddebug = settings.DEBUG
        settings.DEBUG = True

        v1 = Version(name='v1', published=True)
        v1.save(user)

        scenario = app_models.DeviationScenario.objects.get_versionedobject(v_id=1, version=WorkVersion)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)

        scenario = app_models.DeviationScenario.objects.get_versionedobject(v_id=1, version=WorkVersion)

        qu1 = len(connection.queries)
        errors = evaluation.evaluate_causetree_for_deviationscenario(scenario)
        qu2 = len(connection.queries)
        self.assertEquals(qu2 - qu1, 0)

        pubscenario = app_models.DeviationScenario.objects.get_versionedobject(v_id=1, version=v1)

        qu1 = len(connection.queries)
        errors = evaluation.evaluate_causetree_for_deviationscenario(pubscenario)
        qu2 = len(connection.queries)
        self.assertNotEquals(qu2 - qu1, 0)

        settings.DEBUG = olddebug

class Deviations_SearchTestCase(PermissionsMixIn, SettingsMixIn, base_tests.SearchTest):
    searchclasses = {
        'sl_search': app_models.Deviations_SuggestionlistSearch,
        'sc_search': app_models.Deviations_ScenarioSearch,
        'ev_search': app_models.Deviations_CauseSearch,
        'pl_search': app_models.Deviations_ProtectionLayerSearch,
        'me_search': app_models.Deviations_MeasureSearch,
        'apsl_search': app_models.Deviations_APSuggestionlistSearch,
        'ap_search': app_models.Deviations_AttentionPointSearch,
        }


# load extra doc tests

from evaluation import fgt, feq
__test__ = {
    'fgt': fgt,
    'feq': feq,
}

