from analyses.analysisbase import tests as base_tests

from analyses.releases import models as app_models

from django.contrib.auth.models import User

class PermissionsMixIn(object):
    def setPermissions(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('edit_versions','versioning','version'),
            ('display_releasescenarios','releases','releasescenario'),
            ('edit_releasescenarios','releases','releasescenario'),
            ('display_installations','installations','section'),
            ('display_releasesuggestionlists','releases','releasesuggestionlist'),
            ('edit_releasesuggestionlists','releases','releasesuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_releasescenarios','releases','releasescenario'),
            ('display_installations','installations','section'),
            ('display_releasesuggestionlists','releases','releasesuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class SettingsMixIn(object):
    sc_model = app_models.ReleaseScenario
    ev_model = app_models.ReleaseEvent
    pl_model = app_models.ReleaseProtectionLayer
    ap_model = app_models.ReleaseAttentionPoint
    me_model = app_models.ReleaseMeasure
    plme_model = app_models.ReleaseProtectionLayer_Measure
    sl_model = app_models.ReleaseSuggestionlist
    apsl_model = app_models.ReleaseAPSuggestionlist
    tree_model = app_models.ReleaseTreeElement
    namespace = "releases"


class Releases_SuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.SuggestionlistTest):
    pass

class Releases_ScenarioTest(PermissionsMixIn, SettingsMixIn, base_tests.ScenarioTest):
    pass

class Releases_EventTest(PermissionsMixIn, SettingsMixIn, base_tests.EventTest):
    pass

class Releases_ProtectionLayerTest(PermissionsMixIn, SettingsMixIn, base_tests.ProtectionLayerTest):
    pass

class Releases_MeasureTest(PermissionsMixIn, SettingsMixIn, base_tests.MeasureTest):
    pass

class Releases_APSuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.APSuggestionlistTest):
    pass

class Releases_AttentionPointTest(PermissionsMixIn, SettingsMixIn, base_tests.AttentionPointTest):
    pass

class Releases_VersioningTest(PermissionsMixIn, SettingsMixIn, base_tests.VersioningTest):
    pass

class Releases_SearchTestCase(PermissionsMixIn, SettingsMixIn, base_tests.SearchTest):
    searchclasses = {
        'sl_search': app_models.Releases_SuggestionlistSearch,
        'sc_search': app_models.Releases_ScenarioSearch,
        'ev_search': app_models.Releases_EventSearch,
        'pl_search': app_models.Releases_ProtectionLayerSearch,
        'me_search': app_models.Releases_MeasureSearch,
        'apsl_search': app_models.Releases_APSuggestionlistSearch,
        'ap_search': app_models.Releases_AttentionPointSearch,
        }
