from django.contrib.auth.models import User, Group, Permission
from django.test import TestCase

from meetings import models as app_models

import time

from utils import tests as utils_tests


class MeetingsTestCase(TestCase):
    fixtures = ['dummyusers',]

    def setUp(self):
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('edit_versions','versioning','version'),
            ('display_meetings','meetings','meeting'),
            ('edit_meetings','meetings','meeting'),
            ('participate_meetings','meetings','meeting'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_meetings','meetings','meeting'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        s = app_models.Meeting(date='2010-01-01', subject=u'test')
        s.save(User.objects.get(pk=1))

    def test_meeting_views(self):

        base_url = '/meetings/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class meetings(utils_tests.tablelist_TestDefinition):
            description = 'meetings list'
            url = base_url
            users = [1,2,]

            contains = [{'url': base_url + 'add/', 'users': [1,]},]

        class meetings_report_select(utils_tests.TestDefinition):
            description = 'report of list of meetings: select options'
            url = base_url + '?submit_pdf=submit_pdf&filter-subject=e'
            users = [1,2,]

        class meetings_report(utils_tests.postonly_TestDefinition):
            description = 'pdf of list of meetings'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "submit_pdf=submit_pdf&filter-subject=e",
                "submit_pdf": "Generate pdf",
            }

        class meeting(utils_tests.TestDefinition):
            description = 'display meeting'
            url = base_url + '1/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': [1,]},
                {'url': base_url + '1/delete/', 'users': [1,]},
            ]

        class meeting_add(utils_tests.add_TestDefinition):
            description = 'add a meeting'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "date": "2010-01-01",
                "subject": "test",
                "description": "a test",
                "participants": [1,],
                "management-referer": base_url
            }
            redirect_url = base_url

            def posttest(self, testcase, allowed):
                if allowed:
                    app_models.Meeting.objects.order_by('-id')[0].delete(defaultuser)

        class meeting_edit(utils_tests.edit_TestDefinition):
            description = 'edit a meeting'
            url = base_url + '1/edit/'
            users = [1,]
            data = {
                "date": "2010-01-01",
                "subject": "test",
                "description": "a test",
                "participants": [1,],
                "management-referer": base_url + '1/'
            }
            redirect_url = base_url + '1/'

        class meeting_delete(utils_tests.delete_TestDefinition):
            description = 'delete a meeting'
            users = [1,]
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = app_models.Meeting(date='2010-01-01')
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    app_models.Meeting.objects.get(id=self.pk).delete(defaultuser)

        class meeting_report(utils_tests.TestDefinition):
            description = 'pdf report for a meeting'
            url = base_url + '1/pdf/'
            users = [1,2,]

        class meeting_compare(utils_tests.TestDefinition):
            description = 'compare meeting'
            url = base_url + '1/compare/0/'
            users = [1,2,]

        tests = [
            meetings(),
            meetings_report_select(),
            meetings_report(),
            meeting(),
            meeting_add(),
            meeting_edit(),
            meeting_delete(),
            meeting_report(),
            meeting_compare(),
        ]

        utils_tests.run_tests(self, users, tests)

