from django import template

from django.utils.translation import ugettext_lazy as _

from tasks.models import Task
from tasks.views.task import display_url_for_obj, add_url_for_obj

from utils.versioning.models import WorkVersion

register = template.Library()

@register.inclusion_tag('tasks/display_link.html', takes_context=True)
def tasks_link(context, obj):
    """
    Renders a link to the objects tasks list.

    """
    tot_count = Task.objects.tasks_for_object(obj).count()
    open_count = Task.objects.open_tasks_for_object(obj).count()
    late_count = Task.objects.late_tasks_for_object(obj).count()
    if tot_count == 1:
        name = Task._meta.verbose_name
    else:
        name = Task._meta.verbose_name_plural
    user = None
    if context.has_key('user'):
        user = context['user']
    if user and user.has_perm('tasks.edit_tasks'):
        add_url = add_url_for_obj(obj)
    else:
        add_url = u''

    return {
        'MEDIA_URL': context['MEDIA_URL'],
        'display_label': u'%(late_count)s / %(open_count)s / %(tot_count)s %(name)s' % locals(),
        'display_url': display_url_for_obj(obj),
        'add_url': add_url,
    }

@register.tag
def get_tasks(parser, token):
    """
    show first X tasks for user
    example usage:
        {% get_tasks 2 as tasks %}
    """
    bits = token.contents.split()
    if len(bits) != 4:
        raise template.TemplateSyntaxError, "'%s' tag takes four arguments" % bits[0]
    if bits[2] != 'as':
        raise template.TemplateSyntaxError, "Second argument to '%s' tag must be 'as'" % bits[0]
    return GetTasksNode(bits[1], bits[3])

class GetTasksNode(template.Node):
    def __init__(self, num, varname):
        self.num, self.varname = num, varname

    def render(self, context):
        qs = None
        user = None
        if context.has_key('user'):
            user = context['user']
        if user:
            qs = Task.objects.filter(user=user, completiondate__isnull=True
                ).order_by('duedate',).get_versionedobjects(WorkVersion)[:self.num]
        context[self.varname] = qs
        return ''
