"""
custom widgets
"""

from django import forms
from django.conf import settings

from django.forms.widgets import flatatt
from django.utils.encoding import smart_unicode
from django.utils.html import escape
from django.utils.simplejson import JSONEncoder
from django.core.urlresolvers import reverse


# i18n
from django.utils.translation import ugettext_lazy as _

class DateWidget(forms.DateTimeInput):
    """
    like admin date widget with proper date formatting
    uses jquery datepicker
    requires DATE_OUTPUT_FORMAT setting in settings
    """

# TODO: update this and add js and css files

    class Media:
        css = { 'all': (
                settings.MEDIA_URL + u'themes/redmond/jquery-ui-1.8.2.css',
            )}
        js = [
                settings.MEDIA_URL + u'js/jquery-ui-1.8.2.min.js',
                settings.MEDIA_URL + u'js/calendar.js',
            ]

    def __init__(self, attrs={}):
        super(DateWidget, self).__init__(format=settings.DATE_OUTPUT_FORMAT, attrs={'class': 'vDateField', 'size': '10'})

class JQueryAutoCompleteTextInput(forms.TextInput):
    """
    like text input widget with jQuery autocomplete function
    based on djangosnippet 233, see www.djangosnippets.com
    """

    class Media:
        css = { 'all': (
                settings.MEDIA_URL + u'autocomplete/jquery.autocomplete.css',
            )}
        js = [
                settings.MEDIA_URL + u'autocomplete/jquery.autocomplete.js',
            ]

    def __init__(self, source='', namedsource='', options={}, attrs={}):
        """source can be a list containing the autocomplete values or a
        string containing the url used for the XHR request.

        For available options see the autocomplete sample page::
        http://jquery.bassistance.de/autocomplete/"""

        self.options = None
        self.attrs = {'autocomplete': 'off'}
        self.source = source
        self.namedsource = namedsource

        if len(options) > 0:
            self.options = JSONEncoder().encode(options)

        self.attrs.update(attrs)

    def render_js(self, field_id):
        if self.namedsource:
            source = reverse(self.namedsource)
        else:
            source = self.source
        if isinstance(source, list):
            source = JSONEncoder().encode(source)
        elif isinstance(source, str):
            source = "'%s'" % escape(source)
        else:
            raise ValueError('source type is not valid')

        options = ''
        if self.options:
            options += ',%s' % self.options

        return u'$(\'#%s\').autocomplete(%s%s);' % (field_id, source, options)

    def render(self, name, value=None, attrs=None):
        final_attrs = self.build_attrs(attrs, name=name)
        if value:
            final_attrs['value'] = escape(smart_unicode(value))

        if not self.attrs.has_key('id'):
            final_attrs['id'] = 'id_%s' % name

        return u'''<input type="text" %(attrs)s/>
        <script type="text/javascript"><!--//
        %(js)s//--></script>
        ''' % {
            'attrs' : flatatt(final_attrs),
            'js' : self.render_js(final_attrs['id']),
        }

class MarkItUpWidget(forms.Textarea):
    class Media:
        css = {
            'all' : (
                'markitup/skins/simple/style.css',
                'markitup/sets/markdown/style.css',
            )
        }
        js = (
            'markitup/jquery.markitup.js',
            'markitup/sets/markdown/set.js',
        )

    def render(self, name, value, attrs=None):
        html = super(MarkItUpWidget, self).render(name, value, attrs)
        html += u'''<script type="text/javascript">
                 $(document).ready(function() {
                   $("#%(id)s").markItUp(mySettings);
                 });
                 </script>''' % {'id': attrs['id']}
        return html

